<?php

namespace ManiaLivePlugins\Flo\Profiler\Gui\Windows;

use ManiaLive\Event\Dispatcher;
use ManiaLivePlugins\Flo\Profiler\Gui\Controls\NetGraphTab;
use ManiaLivePlugins\Flo\Profiler\Gui\Controls\OverviewTab;
use ManiaLivePlugins\Flo\Profiler\Gui\Controls\MemoryTab;
use ManiaLivePlugins\Flo\Profiler\Gui\Controls\SpeedGraphTab;
use ManiaLive\Gui\Windowing\Controls\Tab;
use ManiaLive\Gui\Windowing\Controls\Tabview;
use ManiaLive\Gui\Windowing\Controls\Frame;
use ManiaLive\Gui\Windowing\Controls\Line;
use ManiaLive\Gui\Windowing\Controls\Panel;
use ManiaLib\Gui\Elements\Bgs1;
use ManiaLib\Gui\Elements\Quad;
use ManiaLib\Gui\Elements\Label;

class Stats extends \ManiaLive\Gui\Windowing\Window
	implements \ManiaLive\Features\Tick\Listener
{
	protected $panel;
	protected $cpu_stats;
	protected $mem_stats;
	protected $max;
	protected $tabview;
	protected $last_refresh;
	public $time_started;
	
	function initializeComponents()
	{
		$this->cpu_stats = array();
		$this->mem_stats = array();
		
		$this->panel = new Panel();
		$this->panel->setTitle('Statistics');
		$this->addComponent($this->panel);
		
		$this->tabview = new Tabview();
		
		// add overview tab
		$tab = new OverviewTab();
		$tab->setTitle('Overview');
		$tab->mem_stats =& $this->mem_stats;
		$tab->cpu_stats =& $this->cpu_stats;
		$this->tabview->addTab($tab);
		
		// add cpu tab
		$tab = new SpeedGraphTab();
		$tab->setTitle('Speed Graph');
		$tab->stats =& $this->cpu_stats;
		$this->tabview->addTab($tab);
		
		// add memory tab
		$tab = new MemoryTab();
		$tab->setTitle('Memory Graph');
		$tab->stats =& $this->mem_stats;
		$this->tabview->addTab($tab);
		
		// add network tab
		$tab = new NetGraphTab();
		$tab->setTitle('Network Graph');
		$tab->net_stats =& $this->net_stats;
		$this->tabview->addTab($tab);
		
		$this->addComponent($this->tabview);
	}
	
	function onShow()
	{
		$this->tabview->getTab(0)->time_started = $this->time_started;
		
		$this->panel->setSize($this->getSizeX(), $this->getSizeY());
		
		$this->tabview->setPosition(1, 6);
		$this->tabview->setSize($this->getSizeX() - 2, $this->getSizeY() - 7);
	}
	
	function onRecover()
	{
		// subscribe to ticker ..
		Dispatcher::register(\ManiaLive\Features\Tick\Event::getClass(), $this);
	}
	
	function onHide()
	{
		// subscribe to ticker ..
		Dispatcher::unregister(\ManiaLive\Features\Tick\Event::getClass(), $this);
	}
	
	function onTick()
	{
		$diff = time() - $this->last_refresh;
		
		switch ($this->tabview->getActiveTabId())
		{
			case 0:
				if ($diff < 1) return;
				$this->show();
				$this->last_refresh = time();
				break;
				
			case 1:
				if ($diff < 1) return;
				$this->last_refresh = time();
				$this->show();
				break;
				
			case 2:
				if ($diff < 3) return;
				$this->last_refresh = time();
				$this->show();
				break;
				
			case 3:
				if ($diff < 1) return;
				$this->last_refresh = time();
				$this->show();
				break;
		}
	}
	
	function addCpuUsage($value)
	{
		$this->cpu_stats[] = $value;
		if (count($this->cpu_stats) > 10) array_shift($this->cpu_stats);
	}
	
	function addMemoryUsage($value)
	{
		$this->mem_stats[] = $value;
		if (count($this->mem_stats) > 10) array_shift($this->mem_stats);
	}
	
	function addNetUsage($receive, $send)
	{
		$this->net_stats[] = array($receive, $send);
		if (count($this->net_stats) > 10) array_shift($this->net_stats);
	}
}

?>