<?php

namespace ManiaLivePlugins\Flo\Profiler;

use ManiaLivePlugins\Flo\Admin\Admin;
use ManiaLive\Features\Admin\AdminGroup;
use ManiaLive\Gui\Windowing\Windows\Info;
use ManiaLive\Config\Loader;
use ManiaLive\Data\Storage;
use ManiaLive\DedicatedApi\Xmlrpc\Client_Gbx;
use ManiaLib\Gui\Elements\Icons128x128_1;
use ManiaLive\Gui\Windowing\Event;
use ManiaLive\Event\Dispatcher;
use ManiaLive\Gui\Windowing\Listener;
use ManiaLivePlugins\Flo\Profiler\Gui\Windows\Stats;
use ManiaLive\DedicatedApi\Connection;

class Profiler extends \ManiaLive\PluginHandler\Plugin
{
	protected $win_stats;
	protected $time_started;
	
	protected $loop_start;
	protected $loop_times;
	protected $loop_count;
	protected $loop_avg;
	
	protected $mem_last_time;
	protected $net_last_time;
	
	const MODE_COUNT = 1;
	const MODE_PROFILER = 2;
	const PROFILER_LOOPS = 1000;
	const MEM_DEFAULT = 134217728;
	
	function onLoad()
	{		
		$this->time_started = time();
		$this->mode = false;
		$this->loop_times = array();
		
		$cmd = $this->registerChatCommand('profile', 'startProfiler', 0, true);
		$cmd->isPublic = false;
		$cmd->help = 'checks the average duration of one application loop.';
		
		$cmd = $this->registerChatCommand('stats', 'showStats', 0, true);
		$cmd->isPublic = false;
		$cmd->help = 'shows statistics on how the application performs.';
		
		if ($this->isPluginLoaded('Flo\Menubar'))
		{
			// set menu icon for dedimanias menu ...
			$this->callPublicMethod('Flo\Menubar',
				'initMenu',
				Icons128x128_1::Statistics);
			
			// add button for records window ...
			$this->callPublicMethod('Flo\Menubar',
				'addButton',
				'Live Statistics',
				array($this, 'showStats'),
				true);
		}
	}
	
	function showStats($login)
	{
		if (!AdminGroup::contains($login))
		{
			$win = Info::Create($login);
			$win->setSize(40, 15);
			$win->setTitle('No Permission');
			$win->setText('The command you have been trying to use is preserved to Administrators!');
			$win->centerOnScreen();
			$win->show();
			return;
		}
		
		$this->mode = self::MODE_COUNT;
		
		$this->loop_start = time();
		$this->win_stats = Stats::Create($login);
		$this->win_stats->time_started = $this->time_started;
		$this->win_stats->setSize(66, 51.5);
		$this->win_stats->centerOnScreen();
		$this->win_stats->show();
		
		// execute profiling on pre and postloop ...
		$this->enableApplicationEvents();
		
		// ... until window is closed
		$this->enableWindowingEvents();
	}
	
	function startProfiler($login)
	{
		if (!AdminGroup::contains($login))
			return Connection::getInstance()->chatSendServerMessage('$f00This command is preserved to Administrators only!');
		
		$this->mode = self::MODE_PROFILER;
		
		// print notification
		Connection::getInstance()->chatSendServerMessage('> Profiling started, this may take a while!', Storage::getInstance()->players[$login]);
		
		// profile on pre and postloop
		$this->enableApplicationEvents();
	}
	
	function onPreLoop()
	{
		switch ($this->mode)
		{
			case self::MODE_COUNT:
				if ($this->loop_count == 0)
					$this->loop_start = microtime(true);
				$this->loop_count++;
				break;
				
			case self::MODE_PROFILER:
				$this->loop_start = microtime(true);
				break;
				
			default:
		}
	}
	
	function onPostLoop()
	{
		switch ($this->mode)
		{
			case self::MODE_COUNT:
				if ($this->loop_start != 0 && microtime(true) - $this->loop_start > 1)
				{
					$this->win_stats->addCpuUsage($this->loop_count);
					$this->loop_count = 0;
					
					$send_diff = Client_Gbx::$sent - $this->net_last_time[1];
					$recv_diff = Client_Gbx::$received - $this->net_last_time[0];
					$this->win_stats->addNetUsage($recv_diff, $send_diff);
					$this->net_last_time = array(Client_Gbx::$received, Client_Gbx::$sent);
				}
				if (microtime(true) - $this->mem_last_time > 3)
				{
					$this->mem_last_time = microtime(true);
					$this->win_stats->addMemoryUsage(memory_get_usage());
				}
				break;
				
			case self::MODE_PROFILER:
				
				$this->loop_count++;
				$this->loop_times[] = microtime(true) - $this->loop_start;
				if ($this->loop_count > self::PROFILER_LOOPS)
				{
					$this->loop_count = 0;
					
					// calculate average loop time
					$this->calculateStats();
					
					// stop profiling
					$this->disableApplicationEvents();
				}
				break;
				
			default:
		}
	}
	
	function onWindowClose($login, $window)
	{
		if ($window == $this->win_stats)
		{
			// stop listening for closing windows
			$this->disableWindowingEvents();
			
			// dont execute onloop events anymore
			$this->disableApplicationEvents();

			$this->win_stats = null;
			$this->loop_count = 0;
		}
	}
	
	function calculateStats()
	{
		// remove first time, its bad!
		array_shift($this->loop_times);
		
		// calculate average ...
		$this->loop_avg = array_sum($this->loop_times) / count($this->loop_times);
		$this->loop_times = array();
		
		$message = 'The average time for one application loop is ' . round($this->loop_avg, 5);
		Connection::getInstance()->chatSendServerMessage($message);
	}
}

?>